#include "utagit.h"
#include "ui_utagit.h"
#include <QtGui>	// defines all qt gui classes

Utagit::Utagit(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::Utagit)
{
    QFile file;
    QByteArray line;
    int n;
    int count,index;
    char *ptr;
    ui->setupUi(this);

    setCurrentPath();
        // connect signals to slots to magically call the functions when buttons are clicked
    connect( ui->actionOpen, SIGNAL( activated() ), this, SLOT( fileOpen() ) );
    connect( ui->actionHelp, SIGNAL( activated() ), this, SLOT( helpHelp() ) );
    connect( ui->actionAbout, SIGNAL( activated() ), this, SLOT( helpAbout() ) );
    connect( ui->pushButton_update, SIGNAL( clicked() ), this, SLOT( update() ) );
    connect( ui->radioButton_edit, SIGNAL(clicked() ), this, SLOT( modeChange() ) );
    connect( ui->radioButton_view, SIGNAL(clicked() ), this, SLOT( modeChange() ) );
    connect( ui->radioButton_update, SIGNAL(clicked() ), this, SLOT( modeChange() ) );
    connect( ui->radioButton_autoUpdate, SIGNAL(clicked() ), this, SLOT( modeChange() ) );
    connect( ui->checkBox_trackUpdate, SIGNAL(clicked() ), this, SLOT( modeChange() ) );
    connect( ui->checkBox_genreUpdate, SIGNAL(clicked() ), this, SLOT( modeChange() ) );
    connect( ui->checkBox_titleUpdate, SIGNAL(clicked() ), this, SLOT( modeChange() ) );
    connect( ui->checkBox_artistUpdate, SIGNAL(clicked() ), this, SLOT( modeChange() ) );
    connect( ui->checkBox_albumUpdate, SIGNAL(clicked() ), this, SLOT( modeChange() ) );
    connect( ui->checkBox_yearUpdate, SIGNAL(clicked() ), this, SLOT( modeChange() ) );
    connect( ui->checkBox_commentUpdate, SIGNAL(clicked() ), this, SLOT( modeChange() ) );
    connect( ui->checkBox_ontop, SIGNAL(clicked() ), this, SLOT( onTop() ) );
    file.setFileName("Genre.txt");
    if (file.open(QIODevice::ReadOnly | QIODevice::Text)) {
      count=0;
      while (!file.atEnd()) {
        line=file.readLine();
        line.chop(1); // remove linefeed from the end
        n=0;
        ptr=line.data();
        while (*ptr==' ' || *ptr=='\t') { // skip whitespace
          ptr++;
        }
        while ((unsigned char)(*ptr-'0')<10) { // convert number
          n=n*10+*ptr++-'0';
        }
        while (*ptr==' ' || *ptr=='\t') { // skip whitespace
          ptr++;
        }
        for (index=0; index<count; index++) {
          if (ui->comboBox_genre->itemText(index)>ptr) {
            break;
          }
        }
        ui->comboBox_genre->insertItem(index,ptr,n);
        count++;
      }
      file.close();
    } else {
      ui->comboBox_genre->addItem("No Genre.txt file found",255);
    }
    modeChange(); // set enables/disables and help text
}

Utagit::~Utagit()
{
    delete ui;
}

// set current path to path of executable file
void Utagit::setCurrentPath(void)
{
    QStringList cla;  // command line arguments

    cla=QCoreApplication::arguments();    // get list of command line arguments (first one is application exe file)
    if (cla.size()>=1) {
          QDir::setCurrent(QFileInfo(cla[0]).absolutePath()); // set current directory to path of executable
    }
}

void Utagit::fileOpen()
{
  QString path; // Qt version of CString
  QFileInfo fileinfo;

  // get a filename using file open dialog box
  path=QFileDialog::getOpenFileName(this,"Select File",QString::null,"*.mp3");
  if (path!="") {
    fileinfo.setFile(path); // get information about file
    if (fileinfo.isFile()) { // if it is a file (not a directory or symlink)
      if (fileinfo.suffix().toLower()=="mp3") { // it must be an mp3 file (all others are ignored)
        readtags(path); //read tags from file
        if (ui->radioButton_autoUpdate->isChecked()) {
          writetags();
        }
      }
    }
  }

}

void Utagit::helpHelp()
{
  QDesktopServices::openUrl(QUrl("UTagIt.html"));
}

void Utagit::helpAbout()
{
    QMessageBox::about(this,"About UTagit","<h1>UTagit 2.00</h1><br>A drag-and-drop MP3 ID3v1.1 tag editor<h2>by Keith Rubow</h2><a href=\"http://www.krubow.com/\">www.krubow.com</a>");

//    QMessageBox::about(this,"About UTagIt",
//    "UTagIt Version 1.00\n"
//    "A drag-and-drop MP3 ID3v1.1 tag editor");
}

void Utagit::update()
{
  writetags();
}

void Utagit::modeChange()
{
  bool edits,enables,incs,update;

  edits=enables=incs=update=0;
  if (ui->radioButton_edit->isChecked()) {
    edits=enables=update=1;
    ui->label_description->setText("Edit selected Tags, Click Update to save");
  }
  if (ui->radioButton_view->isChecked()) {
    ui->label_description->setText("View Tags, no changes allowed");
  }
  if (ui->radioButton_update->isChecked() || ui->radioButton_autoUpdate->isChecked()) {
    edits=enables=incs=update=1;
    if (ui->radioButton_update->isChecked()) {
      ui->label_description->setText("Replace selected tags, Click Update to save");
    } else {
      ui->label_description->setText("Replace selected tags, Updated immediately");
    }
  }
  ui->pushButton_update->setEnabled(update);

  ui->checkBox_trackUpdate->setEnabled(enables);
  ui->checkBox_genreUpdate->setEnabled(enables);
  ui->checkBox_titleUpdate->setEnabled(enables);
  ui->checkBox_artistUpdate->setEnabled(enables);
  ui->checkBox_albumUpdate->setEnabled(enables);
  ui->checkBox_yearUpdate->setEnabled(enables);
  ui->checkBox_commentUpdate->setEnabled(enables);

  ui->lineEdit_track->setEnabled(edits && ui->checkBox_trackUpdate->isChecked());
  ui->comboBox_genre->setEnabled(edits && ui->checkBox_genreUpdate->isChecked());
  ui->lineEdit_title->setEnabled(edits && ui->checkBox_titleUpdate->isChecked());
  ui->lineEdit_artist->setEnabled(edits && ui->checkBox_artistUpdate->isChecked());
  ui->lineEdit_album->setEnabled(edits && ui->checkBox_albumUpdate->isChecked());
  ui->lineEdit_year->setEnabled(edits && ui->checkBox_yearUpdate->isChecked());
  ui->lineEdit_comment->setEnabled(edits && ui->checkBox_commentUpdate->isChecked());

  ui->checkBox_trackInc->setEnabled(incs && ui->checkBox_trackUpdate->isChecked());
  ui->checkBox_titleInc->setEnabled(incs && ui->checkBox_titleUpdate->isChecked());
  ui->checkBox_artistInc->setEnabled(incs && ui->checkBox_artistUpdate->isChecked());
  ui->checkBox_albumInc->setEnabled(incs && ui->checkBox_albumUpdate->isChecked());
  ui->checkBox_commentInc->setEnabled(incs && ui->checkBox_commentUpdate->isChecked());

}

void Utagit::dragEnterEvent(QDragEnterEvent *event)
{
  if (event->mimeData()->hasUrls()) {
    event->acceptProposedAction();
  }
}

void Utagit::dropEvent(QDropEvent *event)
{
  QString filename;
  QFileInfo fileinfo;

  if (event->mimeData()->hasUrls()) {
    foreach (QUrl url, event->mimeData()->urls()) {
      filename=url.toLocalFile();
      fileinfo.setFile(filename); // get information about file
      if (fileinfo.isFile()) { // if it is a file (not a directory or symlink)
        if (fileinfo.suffix().toLower()=="mp3") { // it must be an mp3 file (all others are ignored)
          readtags(url.toLocalFile());
          if (ui->radioButton_autoUpdate->isChecked()) {
            writetags();
          }
        }
      }
    }
  }
}

// read tags from filename (if any) to tagbuffer
// set tagposition to location of tags (or end of file if none)
// transfer only selected tags to edit widgets
void Utagit::readtags(QString filename)
{
  QFile file;
  char buf[31];
  QString str;

  file.setFileName(filename);
  if (file.open(QIODevice::ReadOnly)) {
    currentfile=filename;
    tagposition=file.size();
    memset(tagbuffer,0,128);
    if (tagposition>=128) {
      file.seek(tagposition-128);
      file.read(tagbuffer,128);
    }
    file.close();
    if (strncmp(tagbuffer,"TAG",3)==0) { // if tags exist
      tagposition-=128;
    } else {
      memset(tagbuffer,0,128);
      strncpy(tagbuffer,"TAG",3);
      tagbuffer[127]=(char)255;
    }
    buf[30]=0;
    if (ui->radioButton_edit->isChecked() || ui->radioButton_view->isChecked()) {
      strncpy(buf,tagbuffer+3,30); //title
      ui->lineEdit_title->setText(buf);
      strncpy(buf,tagbuffer+33,30); //artist
      ui->lineEdit_artist->setText(buf);
      strncpy(buf,tagbuffer+63,30); //album
      ui->lineEdit_album->setText(buf);
      strncpy(buf,tagbuffer+93,4); //year
      buf[4]=0;
      ui->lineEdit_year->setText(buf);
      strncpy(buf,tagbuffer+97,29); //comment
      buf[29]=0;
      ui->lineEdit_comment->setText(buf);
      if (tagbuffer[126]==0) { // track
        ui->lineEdit_track->setText("");
      } else {
        str.sprintf("%d",(unsigned)tagbuffer[126]);
        ui->lineEdit_track->setText(str);
      }
      setgenre((unsigned char)tagbuffer[127]);
    }
    if (ui->radioButton_update->isChecked() || ui->radioButton_autoUpdate->isChecked()) {
      if (!ui->checkBox_titleUpdate->isChecked()) {
        strncpy(buf,tagbuffer+3,30); //title
        ui->lineEdit_title->setText(buf);
      }
      if (!ui->checkBox_artistUpdate->isChecked()) {
        strncpy(buf,tagbuffer+33,30); //artist
        ui->lineEdit_artist->setText(buf);
      }
      if (!ui->checkBox_albumUpdate->isChecked()) {
        strncpy(buf,tagbuffer+63,30); //album
        ui->lineEdit_album->setText(buf);
      }
      if (!ui->checkBox_yearUpdate->isChecked()) {
        strncpy(buf,tagbuffer+93,4); //year
        buf[4]=0;
        ui->lineEdit_year->setText(buf);
      }
      if (!ui->checkBox_commentUpdate->isChecked()) {
        strncpy(buf,tagbuffer+97,29); //comment
        buf[29]=0;
        ui->lineEdit_comment->setText(buf);
      }
      if (!ui->checkBox_trackUpdate->isChecked()) {
        if (tagbuffer[126]==0) { // track
          ui->lineEdit_track->setText("");
        } else {
          str.sprintf("%d",(unsigned)tagbuffer[126]);
          ui->lineEdit_track->setText(str);
        }
      }
      if (!ui->checkBox_genreUpdate->isChecked()) {
        setgenre((unsigned char)tagbuffer[127]);
      }
    }
    QFileInfo fileinfo(filename); // this is used for system independant file information
    str="UTagit - " + fileinfo.fileName(); // show name (but not path) in title
    this->setWindowTitle(str);
  }
}

void Utagit::setgenre(int n)
{
  int index,count;

  count=ui->comboBox_genre->count();
  for (index=0; index<count; index++) {
    if (ui->comboBox_genre->itemData(index)==n) {
      ui->comboBox_genre->setCurrentIndex(index);
      return;
    }
  }
  ui->comboBox_genre->setCurrentIndex(0);
}

// transfer only selected tags to tagbuffer
// write tagbuffer to currentfile at position tagposition
// increment any fields enabled for incrementing (update and auto update modes only)
void Utagit::writetags()
{
  char buf[31];
  QString str;
  QFile file;
  int n;

  if (currentfile!="") {
    strncpy(tagbuffer,"TAG",3); // be sure it has proper tag
    if (ui->checkBox_titleUpdate->isChecked()) {
      memset(buf,0,sizeof(buf));
      strncpy(buf,ui->lineEdit_title->text().toAscii(),sizeof(buf));
      memcpy(tagbuffer+3,buf,30);
      if (ui->checkBox_titleInc->isEnabled() && ui->checkBox_titleInc->isChecked()) {
        ui->lineEdit_title->setText(incString(ui->lineEdit_title->text()));
      }
    }
    if (ui->checkBox_artistUpdate->isChecked()) {
      memset(buf,0,sizeof(buf));
      strncpy(buf,ui->lineEdit_artist->text().toAscii(),sizeof(buf));
      memcpy(tagbuffer+33,buf,30);
      if (ui->checkBox_artistInc->isEnabled() && ui->checkBox_artistInc->isChecked()) {
        ui->lineEdit_artist->setText(incString(ui->lineEdit_artist->text()));
      }
    }
    if (ui->checkBox_albumUpdate->isChecked()) {
      memset(buf,0,sizeof(buf));
      strncpy(buf,ui->lineEdit_album->text().toAscii(),sizeof(buf));
      memcpy(tagbuffer+63,buf,30);
      if (ui->checkBox_albumInc->isEnabled() && ui->checkBox_albumInc->isChecked()) {
        ui->lineEdit_album->setText(incString(ui->lineEdit_album->text()));
      }
    }
    if (ui->checkBox_yearUpdate->isChecked()) {
      memset(buf,0,sizeof(buf));
      strncpy(buf,ui->lineEdit_year->text().toAscii(),sizeof(buf));
      memcpy(tagbuffer+93,buf,4);
    }
    if (ui->checkBox_commentUpdate->isChecked()) {
      memset(buf,0,sizeof(buf));
      strncpy(buf,ui->lineEdit_comment->text().toAscii(),sizeof(buf));
      memcpy(tagbuffer+97,buf,29);
      if (ui->checkBox_commentInc->isEnabled() && ui->checkBox_commentInc->isChecked()) {
        ui->lineEdit_comment->setText(incString(ui->lineEdit_comment->text()));
      }
    }
    if (ui->checkBox_trackUpdate->isChecked()) {
      tagbuffer[126]=atoi(ui->lineEdit_track->text().toAscii());
      if (ui->checkBox_trackInc->isEnabled() && ui->checkBox_trackInc->isChecked()) {
        ui->lineEdit_track->setText(incString(ui->lineEdit_track->text()));
      }
    }
    if (ui->checkBox_genreUpdate->isChecked()) {
      n=ui->comboBox_genre->itemData(ui->comboBox_genre->currentIndex()).toInt();
      tagbuffer[127]=(char)n;
    }
    file.setFileName(currentfile);
    if (file.open(QIODevice::ReadWrite)) {
      file.seek(tagposition);
      file.write(tagbuffer,128);
      file.close();
    }
  }
}

QString Utagit::incString(QString str)
{
  int i,len;

  len=str.length();
  for (i=len-1; i>=0; i--) { // look for rightmost digit
    if (str[i].isDigit()) { // found a digit!
      if (str[i]=='9') { // rightmost digit is 9
        return incString(str.left(i))+"0"+str.right(len-i-1); // increment remaining string, and replace 9 with 0
      } else {
        str[i]=QChar(str[i].toAscii()+1); // rightmost digit<9, inc the digit
        return str;
      }
    }
  }
  return str+"1"; // no digit, concatinate a 1
}

void Utagit::onTop()
{
  int xpos,ypos;

  xpos=x();
  ypos=y();
  if (ui->checkBox_ontop->isChecked()) {
    setWindowFlags(windowFlags() | Qt::WindowStaysOnTopHint);
  } else {
    setWindowFlags(windowFlags() & ~Qt::WindowStaysOnTopHint);
  }
  move(xpos,ypos);
  show();
}
